kernel void nbody(global float* p, global float* pNeu, global float* v, const float DELTA_T, const float EPSILON_SQUARED)  
{
    uint id = get_global_id(0);
    uint N = get_global_size(0);
    
    uint elems = 3;

    float px = p[elems * id + 0];
    float py = p[elems * id + 1];
    float pz = p[elems * id + 2];
    
    float ax = 0;
    float ay = 0;
    float az = 0; 
    
    for(uint i = 0; i < N; ++i)
    {  
        float opx = p[elems * i + 0];
        float opy = p[elems * i + 1];
        float opz = p[elems * i + 2];
    
        float rx = opx - px;
        float ry = opy - py;
        float rz = opz - pz;

        float distSqr = rx * rx + ry * ry + rz * rz + EPSILON_SQUARED;  

        float invDist = rsqrt(distSqr);
        float invDistCube = invDist * invDist * invDist;

        ax += rx * invDistCube;
        ay += ry * invDistCube;  
        az += rz * invDistCube;  
    }

    float vx = v[elems * id + 0];
    float vy = v[elems * id + 1];
    float vz = v[elems * id + 2];
    
    vx += ax * DELTA_T;
    vy += ay * DELTA_T;
    vz += az * DELTA_T;
    
    px += vx * DELTA_T;
    py += vy * DELTA_T;
    pz += vz * DELTA_T;
    
    v[elems * id + 0] = vx;
    v[elems * id + 1] = vy;
    v[elems * id + 2] = vz;
    
    pNeu[elems * id + 0] = px;
    pNeu[elems * id + 1] = py;
    pNeu[elems * id + 2] = pz;
}